#! /usr/bin/python
#
# This is a translator so that I2C devices can be connected to the serial bus
# It uses the BV4221_V2 to do the translation.
# The protocol used is that same as that for notsmb:
# http://www.pichips.co.uk/index.php/RPi_Not_smBUS
# Which was specially written for the RPi. The idea is that anything written
# for i2c using this module will be 100% compatable with the RPi i2c.
#
# BEFORE USING
# The BV4221 needs to be attached to a console first and the following commands
# are required for use with this software
# Make sure that the BV4221_V2 has firmware version 2.5
# if not upgrade see http://doc.byvac.com/index.php5?title=Product_BV4221
# P0    // tuns off the prompt
# D     // sets decimal
# F     // fixes this behaviour
# B115200 // fixes baud rate
#
# Example of use:
# When using notsmb:
# from notsmb import notSMB
# bus = notSMB(0) # set to i2c channel
#
# When using byi2c:
# from bv4221_i2c import By2cbus
# bus = By2cbus("COM21",115200) # specify com port and baud rate
#
# Both are used in the same way so code is intergchangable e.g.
# list = bus.detect()
#
import serial
from time import sleep
#

class   By2cbus:
    """Common entry point for I2C, emulates subset the notsmb"""
    """As an example
       import notsmb
       bus = notsm.notSMB(0)
       OR
       import bvbus
       bus = bvbus.bvbus(port,baud)
    """
    sp = None   # sp is set to communication device by Init()
    ACK = "*"
    def __init__(self,port,baud):
        try:
            self.sp=serial.Serial(port, baud, timeout=.05, stopbits=1, parity='N' )
            sleep(1)  # first output after connext is ACK  
        except:
            msg = "Cant open COM port: "+port
            dlg = wx.MessageDialog(None, msg, "COM Error", wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy() 
    # --------------------------------------------------------------------------
    # This is the only function needed for reading and writing to BV4221. It is
    # similar to spi in that there is always an input and output even if one
    # or the other is not required.
    # Use:
    # The cmd must be the full command including the address and \r at the end
    # Example: s = sv3(sp,"s 1 1 p\r") # 
    # Returns: a list
    # --------------------------------------------------------------------------
    def bvi2c(self,cmd):
        k = ''
        v = ''
        # send command
        self.sp.flushInput()
        self.sp.write(cmd)
        #print "send ",cmd # debug
        # return value
        while k != self.ACK:
            k = self.sp.read(1)
            if k == self.ACK: # all done
                if len(v) == 0:
                    v = self.ACK
                break
            else:
                if len(k) == 0:
                    break
                else:
                    v = v + k
        #convert sting into list
        rvl = []
        if len(v) != 0:
            ss = v.rsplit(',')
            for s in ss:
                try:
                    rvl.append(int(s))
                except:
                    rvl = [] # clear list on any error
        return rvl

    # --------------------------------------------------------------------------
    # returns a list of devices
    # --------------------------------------------------------------------------
    def detect(self):
        l = self.bvi2c("x\r") # bv4221 returns 8 bit address
        rv = []
        if len(l) > 0:
            for n in l:
                try:
                    rv.append(n/2)
                except:
                    return []
        return rv

    # --------------------------------------------------------------------------
    # list = bus.i2c(adr,[wtiteList],<bytes to read>)
    # --------------------------------------------------------------------------
    def i2c(self,adr,listWrite,nRead):
        adr *= 2 # need 8 bit address
        wl = ""
        rvList = [] # may return an empty list
        # see if there is anything to write
        if len(listWrite) == 0:
            # this is just read so address needs to be + 1
            adr += 1
            rvList = self.bvi2c("s-"+str(adr)+" g-"+str(nRead)+" p\r")
        else:    
            #convert write list to a string
            for n in listWrite:
                wl += str(n)+" "
            if nRead == 0:
                # just write
                rvList = self.bvi2c("s-"+str(adr)+" "+wl+" p\r")
            else:
                # write then read
                rvList = self.bvi2c("s-"+str(adr)+" "+wl+"r g-"+str(nRead)+" p\r")
        return rvList


