#!/usr/bin/env python
# BV4601

from sv3common import sv3, util_error, util_information
from time import sleep

# Global
EEsettings = [] # saves changes for writing to ee[rpm
VoltRef = {1:'1.024V',2:'2.048V',3:'4.096V',4:'VDD'}
i2cadr = None
bus = None

ACK = '\006'
NACK = '\025'

# ------------------------------------------------------------------------------
# start up and close down
# Load frame with default values, from EEPROM
# ------------------------------------------------------------------------------
def onActivate(wnd):
    global EEsettings
    global i2cadr
    global bus
    i2cadr = int(wnd.adr)
    bus = wnd.sp
    # update status bar
    wnd.lblDevice.SetLabel(wnd.barStatus.GetStatusText(0))
    # remove com and baud
    wnd.barStatus.SetStatusText("",1)
    wnd.barStatus.SetStatusText("",2)
    # device ID
    dev = bus.i2c(i2cadr,[0xa1],2)
    v = dev[0]*256
    v += dev[1]
    wnd.barStatus.SetStatusText(str(v),3)
    wnd.barStatus.SetStatusText(str(i2cadr),3) # address
    
    # load eeprom values to 16
    EEsettings = []
    for j in range(16,32):
        v = bus.i2c(i2cadr,[0x90,j],1)
        EEsettings.append(str(v[0]))
        
    # 16(L) & 17(H) is scale factor
    v = int(EEsettings[1])*256
    v += int(EEsettings[0])
    wnd.txtScale.SetValue(str(v))
    # 18 is Vref
    wnd.spnVolt.SetValue(int(EEsettings[2]))
    onVolt(wnd) # update lable
    # 19 is ADC Scan rate in mS
    wnd.txtAdcscan.SetValue(EEsettings[3])

# Close is important as this is a modal frame
def onClose(wnd,event):
    wnd.MakeModal(False)
    event.Skip()
    
# ******************************************************************************
# ******************************************************************************
# EVENTS

# ------------------------------------------------------------------------------
def onSoftreset(wnd):
    bus.i2c(i2cadr,[0x95],0)
    wnd.barStatus.SetStatusText("Reset complete")

# ------------------------------------------------------------------------------
def onHardreset(wnd):
    wnd.sp.SetDTR(True)
    sleep(0.1)
    wnd.sp.SetDTR(False)
    wnd.barStatus.SetStatusText("Reset complete")

# ******************************************************************************
# 
# ******************************************************************************
# ------------------------------------------------------------------------------
# Scale button adjusts timer scale value
def onScale(wnd):
    v = int(wnd.txtScale.GetValue())
    # needs to be split into high and low bytes
    EEsettings[1] = str(v / 256) # high
    EEsettings[0] = str(v % 256) # low
    bus.i2c(i2cadr,[0x91,16,int(EEsettings[0])],0)
    #i2c_twobytes(bus,i2cadr,16,int(EEsettings[0]))
    #sv3(wnd.sp,wnd.adr+"W16,"+EEsettings[0]+"\r") # write value to eeprom
    sleep(0.1)
    bus.i2c(i2cadr,[0x91,17,int(EEsettings[1])],0)
    #sv3(wnd.sp,wnd.adr+"W17,"+EEsettings[1]+"\r") # write value to eeprom
    
# ------------------------------------------------------------------------------
# relay set on/off and time for A
def onSeta(wnd):
    t = int(wnd.txtDelaya.GetValue())
    th = t / 256
    tl = t % 256
    if wnd.rdoA.GetSelection() == 0:
        list = [1,1,th,tl]
    else:
        list = [1,0,th,tl]
    bus.i2c(i2cadr,list,0)

# ------------------------------------------------------------------------------
# relay set on/off and time for B
def onSetb(wnd):
    t = int(wnd.txtDelayb.GetValue())
    th = t / 256
    tl = t % 256
    if wnd.rdoB.GetSelection() == 0:
        list = [2,1,th,tl]
    else:
        list = [2,0,th,tl]
    bus.i2c(i2cadr,list,0)

# ------------------------------------------------------------------------------
# Helper updates all text boxes
def UpdateTextBoxes(wnd):
        vv = bus.i2c(i2cadr,[5,1],2)
        v = vv[0]*256
        v += vv[1]
        wnd.txtAN1.SetValue(str(v))

        vv = bus.i2c(i2cadr,[5,2],2)
        v = vv[0]*256
        v += vv[1]
        wnd.txtAN2.SetValue(str(v))

        vv = bus.i2c(i2cadr,[5,3],2)
        v = vv[0]*256
        v += vv[1]
        wnd.txtAN3.SetValue(str(v))

        vv = bus.i2c(i2cadr,[5,4],2)
        v = vv[0]*256
        v += vv[1]
        wnd.txtAN4.SetValue(str(v))
        
# ------------------------------------------------------------------------------
# Updates text boxes
def onUpdate(wnd):
        UpdateTextBoxes(wnd)

# ------------------------------------------------------------------------------
# Sets scan rate for adc
def onAdcscan(wnd):
    v = int(wnd.txtAdcscan.GetValue())
    if v > 255:
        v = 255
    if v < 0:
        v = 0
    if v == 0:
        util_information("0 will set the dealy to 256 mS")
    EEsettings[3] = str(v)
    bus.i2c(i2cadr,[0x91,19,int(EEsettings[3])],0)
    #sv3(wnd.sp,wnd.adr+"W19,"+EEsettings[3]+"\r") # write value to eeprom
    
# ------------------------------------------------------------------------------
# Digital channel 1
def onD1(wnd):
    if wnd.rdoD1.GetSelection() == 0:
        bus.i2c(i2cadr,[8,0],0)
        #sv3(wnd.sp,wnd.adr+"d0\r")
    else:
        bus.i2c(i2cadr,[8,1],0)
        #sv3(wnd.sp,wnd.adr+"d1\r")

# ------------------------------------------------------------------------------
# spinner update
def onVolt(wnd):
    # update lable
    vr = wnd.spnVolt.GetValue()
    wnd.lblVoltage.SetLabel(VoltRef[vr])
    # update device
    bus.i2c(i2cadr,[7,int(vr)],1)

# ------------------------------------------------------------------------------
# Scan rate buttond
def onScan(wnd):
    EEsettings[0] = wnd.txtScan.GetValue() # get new rate
    bus.i2c(i2cadr,[0x91,16,int(EEsettings[0])],0)
    #sv3(wnd.sp,wnd.adr+"W16,"+EEsettings[0]+"\r") # write to EEPROM
    onSoftreset(wnd)
    

"""
class bv4601(wx.Frame):
    def __init__(self, type, wnd, com, *args, **kwds):
        # begin wxGlade: pi11.__init__
        kwds["style"] = wx.DEFAULT_FRAME_STYLE
        wx.Frame.__init__(self, *args, **kwds)

        if type == 'I':
            import bv4601_Ievents as events
        if type == 'S':
            import bv4601_events as events
        self.evt = events
        self.sp = com
        self.wnd = wnd
        self.adr = wnd.barStatus.GetStatusText(5)

        # Menu Bar

##binds
        self.Bind(wx.EVT_ACTIVATE, self.evt.onActivate(self))
        self.Bind(wx.EVT_CLOSE, self.onClose)

## events
    def onClose(self, event):  # wxGlade: bv4603.<event_handler>
        self.evt.onClose(self,event)

    def onSoftreset(self, event):  # wxGlade: bv4603.<event_handler>
        self.evt.onSoftreset(self)

    def onHardreset(self, event):  # wxGlade: bv4603.<event_handler>
        self.evt.onHardreset(self)

    def onScale(self, event):  # wxGlade: bv4601.<event_handler>
        self.evt.onScale(self)

    def onSeta(self, event):  # wxGlade: bv4601.<event_handler>
        self.evt.onSeta(self)

    def onSetb(self, event):  # wxGlade: bv4601.<event_handler>
        self.evt.onSetb(self)

    def onUpdate(self, event):  # wxGlade: bv4601.<event_handler>
        self.evt.onUpdate(self)

    def onVolt(self, event):  # wxGlade: bv4601.<event_handler>
        self.evt.onVolt(self)

    def onD1(self, event):  # wxGlade: bv4601.<event_handler>
        self.evt.onD1(self)

    def onAdcscan(self, event):  # wxGlade: bv4601.<event_handler>
        self.evt.onAdcscan(self)

"""
