#!/usr/bin/env python
# P011

from sv3common import sv3, util_error, util_information
from time import sleep

# Global
EEsettings = [] # saves changes for writing to ee[rpm
VoltRef = {0:'Vref Pin',1:'1.024V',2:'2.048V',3:'4.096V',4:'VDD'}

ACK = '\006'
NACK = '\025'

# ------------------------------------------------------------------------------
# start up and close down
# Load frame with default values, from EEPROM
# ------------------------------------------------------------------------------
def onActivate(wnd):
    global EEsettings
    global i2cadr
    global bus
    wnd.lblDevice.SetLabel('Device ('+wnd.adr+')')
    i2cadr = int(wnd.adr)
    bus = wnd.sp
    # update status bar
    wnd.lblDevice.SetLabel(wnd.barStatus.GetStatusText(0))
    # remove com and baud
    wnd.barStatus.SetStatusText("",1)
    wnd.barStatus.SetStatusText("",2)
    # device ID
    dev = bus.i2c(i2cadr,[0xa1],2)
    v = dev[0]*256
    v += dev[1]
    wnd.barStatus.SetStatusText(str(v),3)
    wnd.barStatus.SetStatusText(str(i2cadr),3) # address
    
    # load eeprom values to 16
    EEsettings = []
    for j in range(16,32):
        v = bus.i2c(i2cadr,[0x90,j],1)
        EEsettings.append(str(v[0]))
    
    # 16 is scan rate
    wnd.txtScan.SetValue(EEsettings[0])
    # 17 is Vref
    wnd.spnVolt.SetValue(int(EEsettings[1]))
    onVolt(wnd) # update lable

# Close is important as this is a modal frame
def onClose(wnd,event):
    wnd.MakeModal(False)
    event.Skip()
    
# ******************************************************************************
# ******************************************************************************
# EVENTS

# ------------------------------------------------------------------------------
def onSoftreset(wnd):
    bus.i2c(i2cadr,[0x95],0)
    wnd.barStatus.SetStatusText("Reset complete")

# ------------------------------------------------------------------------------
def onHardreset(wnd):
    wnd.sp.SetDTR(True)
    sleep(0.1)
    wnd.sp.SetDTR(False)
    wnd.barStatus.SetStatusText("Reset complete")

# ******************************************************************************
# 
# ******************************************************************************
#
def getADCvalue(chan):
        vl = bus.i2c(i2cadr,[1,chan],2)
        #print vl
        v = int(vl[0])*256
        v += int(vl[1])
        return str(v)

# ------------------------------------------------------------------------------
# Helper updates all text boxes
def UpdateTextBoxes(wnd):
        v = getADCvalue(1)
        wnd.txtAN1.SetValue(v)
        v = getADCvalue(2)
        wnd.txtAN2.SetValue(v)
        v = getADCvalue(3)
        wnd.txtAN3.SetValue(v)
        v = getADCvalue(4)
        wnd.txtAN4.SetValue(v)
        v = getADCvalue(5)
        wnd.txtAN5.SetValue(v)
        v = getADCvalue(6)
        wnd.txtAN6.SetValue(v)
        v = getADCvalue(7)
        wnd.txtAN7.SetValue(v)
        v = getADCvalue(8)
        wnd.txtAN8.SetValue(v)
        v = getADCvalue(9)
        wnd.txtAN9.SetValue(v)
        
# ------------------------------------------------------------------------------
# Updates text boxes
def onUpdate(wnd):
        UpdateTextBoxes(wnd)

# ------------------------------------------------------------------------------
# spinner update
def onVolt(wnd):
    # update lable
    vr = wnd.spnVolt.GetValue()
    wnd.lblVoltage.SetLabel(VoltRef[vr])
    # update device
    bus.i2c(i2cadr,[3,vr],0)

# ------------------------------------------------------------------------------
# Scan rate buttond
def onScan(wnd):
    EEsettings[0] = wnd.txtScan.GetValue() # get new rate
    bus.i2c(i2cadr,[0x91,16,int(EEsettings[0])],0)
    #sv3(wnd.sp,wnd.adr+"W16,"+EEsettings[0]+"\r") # write to EEPROM
    onSoftreset(wnd)
    
# ------------------------------------------------------------------------------
# Digital channel 1
def onD1(wnd):
    if wnd.rdoD1.GetSelection() == 0:
        bus.i2c(i2cadr,[2,1,0],0)
    else:
        bus.i2c(i2cadr,[2,1,1],0)
            
# ------------------------------------------------------------------------------
# Digital channel 2
def onD2(wnd):
    if wnd.rdoD2.GetSelection() == 0:
        bus.i2c(i2cadr,[2,2,0],0)
    else:
        bus.i2c(i2cadr,[2,2,1],0)

# ------------------------------------------------------------------------------
# Digital channel 2
def onD3(wnd):
    if wnd.rdoD3.GetSelection() == 0:
        bus.i2c(i2cadr,[2,3,0],0)
    else:
        bus.i2c(i2cadr,[2,3,1],0)

"""
class p011(wx.Frame):
    def __init__(self, type, wnd, com, *args, **kwds):
        # begin wxGlade: pi11.__init__
        kwds["style"] = wx.DEFAULT_FRAME_STYLE
        wx.Frame.__init__(self, *args, **kwds)

        if type == 'I':
            import p011_Ievents as events
        if type == 'S':
            import p011_events as events
        self.evt = events
        self.sp = com
        self.wnd = wnd
        self.adr = wnd.barStatus.GetStatusText(5)

        # Menu Bar

##binds
        self.Bind(wx.EVT_ACTIVATE, self.evt.onActivate(self))
        self.Bind(wx.EVT_CLOSE, self.onClose)

## events
    def onClose(self, event):  # wxGlade: bv4603.<event_handler>
        self.evt.onClose(self,event)

    def onSoftreset(self, event):  # wxGlade: bv4603.<event_handler>
        self.evt.onSoftreset(self)

    def onHardreset(self, event):  # wxGlade: bv4603.<event_handler>
        self.evt.onHardreset(self)

    def onUpdate(self, event):  # wxGlade: pi11.<event_handler>
        self.evt.onUpdate(self)

    def onVolt(self, event):  # wxGlade: pi11.<event_handler>
        self.evt.onVolt(self)

    def onScan(self, event):  # wxGlade: p011.<event_handler>
        self.evt.onScan(self)

    def onD1(self, event):  # wxGlade: pi11.<event_handler>
        self.evt.onD1(self)

    def onD2(self, event):  # wxGlade: pi11.<event_handler>
        self.evt.onD2(self)

    def onD3(self, event):  # wxGlade: pi11.<event_handler>
        self.evt.onD3(self)

"""
