#!/usr/bin/env python
# P011

from sv3common import sv3, util_error, util_information
from time import sleep

# Global
EEsettings = [] # saves changes for writing to ee[rpm
VoltRef = {0:'Vref Pin',1:'1.024V',2:'2.048V',3:'4.096V',4:'VDD'}

ACK = '\006'
NACK = '\025'

# ------------------------------------------------------------------------------
# start up and close down
# Load frame with default values, from EEPROM
# ------------------------------------------------------------------------------
def onActivate(wnd):
    global EEsettings
    wnd.lblDevice.SetLabel('Device ('+wnd.adr+')')
    # update status bar
    wnd.barStatus.SetStatusText(str(wnd.sp.port),1)
    wnd.barStatus.SetStatusText(str(wnd.sp.baudrate),2)
    s = sv3(wnd.sp,wnd.adr+"D\r") # device ID
    wnd.barStatus.SetStatusText(s,3)
    s =sv3(wnd.sp,wnd.adr+"V\r") # version
    wnd.barStatus.SetStatusText(s,4)
    wnd.barStatus.SetStatusText(wnd.adr,5)
    
    s = sv3(wnd.sp, wnd.adr+"R16,9\r")
    EEsettings = s.rsplit(",")
    
    # 16 is scan rate
    wnd.txtScan.SetValue(EEsettings[0])
    # 17 is Vref
    wnd.spnVolt.SetValue(int(EEsettings[1]))
    onVolt(wnd) # update lable

# Close is important as this is a modal frame
def onClose(wnd,event):
    wnd.MakeModal(False)
    event.Skip()
    
# ******************************************************************************
# ******************************************************************************
# EVENTS

# ------------------------------------------------------------------------------
def onSoftreset(wnd):
    sv3(wnd.sp,wnd.adr+"C\r")
    wnd.barStatus.SetStatusText("Reset complete")

# ------------------------------------------------------------------------------
def onHardreset(wnd):
    wnd.sp.SetDTR(True)
    sleep(0.1)
    wnd.sp.SetDTR(False)
    wnd.barStatus.SetStatusText("Reset complete")

# ******************************************************************************
# 
# ******************************************************************************
# ------------------------------------------------------------------------------
# Helper updates all text boxes
def UpdateTextBoxes(wnd):
        v = sv3(wnd.sp,wnd.adr+"a1\r")
        wnd.txtAN1.SetValue(v)
        v = sv3(wnd.sp,wnd.adr+"a2\r")
        wnd.txtAN2.SetValue(v)
        v = sv3(wnd.sp,wnd.adr+"a3\r")
        wnd.txtAN3.SetValue(v)
        v = sv3(wnd.sp,wnd.adr+"a4\r")
        wnd.txtAN4.SetValue(v)
        v = sv3(wnd.sp,wnd.adr+"a5\r")
        wnd.txtAN5.SetValue(v)
        v = sv3(wnd.sp,wnd.adr+"a6\r")
        wnd.txtAN6.SetValue(v)
        v = sv3(wnd.sp,wnd.adr+"a7\r")
        wnd.txtAN7.SetValue(v)
        v = sv3(wnd.sp,wnd.adr+"a8\r")
        wnd.txtAN8.SetValue(v)
        v = sv3(wnd.sp,wnd.adr+"a9\r")
        wnd.txtAN9.SetValue(v)
        
# ------------------------------------------------------------------------------
# Updates text boxes
def onUpdate(wnd):
        UpdateTextBoxes(wnd)

# ------------------------------------------------------------------------------
# spinner update
def onVolt(wnd):
    # update lable
    vr = wnd.spnVolt.GetValue()
    wnd.lblVoltage.SetLabel(VoltRef[vr])
    # update device
    sv3(wnd.sp,wnd.adr+"v"+str(vr)+"\r")

# ------------------------------------------------------------------------------
# Scan rate buttond
def onScan(wnd):
    EEsettings[0] = wnd.txtScan.GetValue() # get new rate
    sv3(wnd.sp,wnd.adr+"W16,"+EEsettings[0]+"\r") # write to EEPROM
    onSoftreset(wnd)
    
# ------------------------------------------------------------------------------
# Digital channel 1
def onD1(wnd):
    if wnd.rdoD1.GetSelection() == 0:
        sv3(wnd.sp,wnd.adr+"d1,0\r")
    else:
        sv3(wnd.sp,wnd.adr+"d1,1\r")
    
# ------------------------------------------------------------------------------
# Digital channel 2
def onD2(wnd):
    if wnd.rdoD2.GetSelection() == 0:
        sv3(wnd.sp,wnd.adr+"d2,0\r")
    else:
        sv3(wnd.sp,wnd.adr+"d2,1\r")

# ------------------------------------------------------------------------------
# Digital channel 2
def onD3(wnd):
    if wnd.rdoD3.GetSelection() == 0:
        sv3(wnd.sp,wnd.adr+"d3,0\r")
    else:
        sv3(wnd.sp,wnd.adr+"d3,1\r")

"""
class p011(wx.Frame):
    def __init__(self, wnd, com, *args, **kwds):
        # begin wxGlade: pi11.__init__
        kwds["style"] = wx.DEFAULT_FRAME_STYLE
        wx.Frame.__init__(self, *args, **kwds)

        self.sp = com
        self.wnd = wnd
        self.adr = wnd.barStatus.GetStatusText(5)

        # Menu Bar

##binds
        self.Bind(wx.EVT_ACTIVATE, self.evt.onActivate(self))
        self.Bind(wx.EVT_CLOSE, self.onClose)

## events
    def onClose(self, event):  # wxGlade: bv4603.<event_handler>
        evt.onClose(self,event)

    def onSoftreset(self, event):  # wxGlade: bv4603.<event_handler>
        evt.onSoftreset(self)

    def onHardreset(self, event):  # wxGlade: bv4603.<event_handler>
        evt.onHardreset(self)

    def onUpdate(self, event):  # wxGlade: pi11.<event_handler>
        evt.onUpdate(self)

    def onVolt(self, event):  # wxGlade: pi11.<event_handler>
        evt.onVolt(self)

    def onScan(self, event):  # wxGlade: p011.<event_handler>
        evt.onScan(self)

    def onD1(self, event):  # wxGlade: pi11.<event_handler>
        evt.onD1(self)

    def onD2(self, event):  # wxGlade: pi11.<event_handler>
        evt.onD2(self)

    def onD3(self, event):  # wxGlade: pi11.<event_handler>
        evt.onD3(self)

"""
