#!/usr/bin/env python
# P011

from sv3common import sv3, util_error, util_information
from time import sleep
import wx # for grid cell change

# Global
EEsettings = [] # saves changes for writing to ee[rpm

ACK = '\006'
NACK = '\025'

# ------------------------------------------------------------------------------
# start up and close down
# Load frame with default values, from EEPROM
# ------------------------------------------------------------------------------
def onActivate(wnd):
    global EEsettings
    wnd.lblDevice.SetLabel('Device ('+wnd.adr+')')
    # update status bar
    wnd.barStatus.SetStatusText(str(wnd.sp.port),1)
    wnd.barStatus.SetStatusText(str(wnd.sp.baudrate),2)
    s = sv3(wnd.sp,wnd.adr+"D\r") # device ID
    wnd.barStatus.SetStatusText(s,3)
    s =sv3(wnd.sp,wnd.adr+"V\r") # version
    wnd.barStatus.SetStatusText(s,4)
    wnd.barStatus.SetStatusText(wnd.adr,5)
    
    s = sv3(wnd.sp, wnd.adr+"R16,9\r")
    EEsettings = s.rsplit(",")
    
    # Update I/O grid with default input values
    for col in range(0,8): # 0 to 7!!
        wnd.grdSet.SetCellValue(0,col,'1' )
    for col in range(0,8):
        wnd.grdWrite.SetCellValue(0,col,'0' )
    
    # Get atual values for read
    onRead(wnd)
        

# Close is important as this is a modal frame
def onClose(wnd,event):
    wnd.MakeModal(False)
    event.Skip()
    
# ******************************************************************************
# ******************************************************************************
# EVENTS

# ------------------------------------------------------------------------------
def onSoftreset(wnd):
    sv3(wnd.sp,wnd.adr+"C\r")
    wnd.barStatus.SetStatusText("Reset complete")

# ------------------------------------------------------------------------------
def onHardreset(wnd):
    wnd.sp.SetDTR(True)
    sleep(0.1)
    wnd.sp.SetDTR(False)
    wnd.barStatus.SetStatusText("Reset complete")

# ******************************************************************************
# 
# ******************************************************************************
# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------
def onCellchangeSet(wnd,event):
    row = event.GetRow()
    col = event.GetCol()
    wnd.grdSet.SetCellTextColour(row, col, wx.RED)


def onCellchangeWrite(wnd,event):
    row = event.GetRow()
    col = event.GetCol()
    wnd.grdWrite.SetCellTextColour(row, col, wx.RED)

# ------------------------------------------------------------------------------
# Read 8 bits into grid
def onRead(wnd):
    col = 7 # backwards
    for n in range(1,9): # 1 to 8
        v = sv3(wnd.sp,wnd.adr+"i"+str(n)+"\r");
        wnd.grdRead.SetCellValue(0,col,v)
        col -= 1
        
# ------------------------------------------------------------------------------
# Set I/O values
def onSet(wnd):
    col = 7 # backwards
    for n in range(1,9): # 1 to 8
        v = wnd.grdSet.GetCellValue(0,col)
        sv3(wnd.sp,wnd.adr+"s"+str(n)+","+v+"\r")
        wnd.grdSet.SetCellTextColour(0, col, wx.BLACK)
        col -= 1
    wnd.grdSet.ForceRefresh()

# ------------------------------------------------------------------------------
# Write values
def onWrite(wnd):
    col = 7 # backwards
    for n in range(1,9): # 1 to 8
        v = wnd.grdWrite.GetCellValue(0,col)
        sv3(wnd.sp,wnd.adr+"p"+str(n)+","+v+"\r")
        wnd.grdWrite.SetCellTextColour(0, col, wx.BLACK)
        col -= 1
    wnd.grdWrite.ForceRefresh()

# ------------------------------------------------------------------------------
# PWM1
def onPwm1(wnd):
    v = wnd.sldPwm1.GetValue()
    sv3(wnd.sp,wnd.adr+"m1,"+str(v)+"\r")
# PWM2
def onPwm2(wnd):
    v = wnd.sldPwm2.GetValue()
    sv3(wnd.sp,wnd.adr+"m2,"+str(v)+"\r")
# PWM3
def onPwm3(wnd):
    v = wnd.sldPwm3.GetValue()
    sv3(wnd.sp,wnd.adr+"m3,"+str(v)+"\r")
# PWM4
def onPwm4(wnd):
    v = wnd.sldPwm4.GetValue()
    sv3(wnd.sp,wnd.adr+"m4,"+str(v)+"\r")

"""
class p015(wx.Frame):
    def __init__(self, type, wnd, com, *args, **kwds):
        # begin wxGlade: pi11.__init__
        kwds["style"] = wx.DEFAULT_FRAME_STYLE
        wx.Frame.__init__(self, *args, **kwds)

        if type == 'I':
            import p015_Ievents as events
        if type == 'S':
            import p015_events as events
        self.evt = events
        self.sp = com
        self.wnd = wnd
        self.adr = wnd.barStatus.GetStatusText(5)

        # Menu Bar

## Other
        self.grdSet.SetRowLabelValue(0, _(">>"))
        self.grdRead.SetRowLabelValue(0, _(">>"))
        self.grdWrite.SetRowLabelValue(0, _(">>"))
        # end wxGlade

##binds
        self.Bind(wx.EVT_ACTIVATE, self.evt.onActivate(self))
        self.Bind(wx.EVT_CLOSE, self.onClose)

## events
    def onClose(self, event):  # wxGlade: bv4603.<event_handler>
        self.evt.onClose(self,event)

    def onSoftreset(self, event):  # wxGlade: bv4603.<event_handler>
        self.evt.onSoftreset(self)

    def onHardreset(self, event):  # wxGlade: bv4603.<event_handler>
        self.evt.onHardreset(self)

    def onSet(self, event):  # wxGlade: p015.<event_handler>
        self.evt.onSet(self)

    def onCellchangeSet(self, event):  # wxGlade: p015.<event_handler>
        self.evt.onCellchangeSet(self,event)
        
    def onRead(self, event):  # wxGlade: p015.<event_handler>
        self.evt.onRead(self)

    def onWrite(self, event):  # wxGlade: p015.<event_handler>
        self.evt.onWrite(self)

    def onCellchangeWrite(self, event):  # wxGlade: p015.<event_handler>
        self.evt.onCellchangeWrite(self,event)


    def onPwm1(self, event):  # wxGlade: p015.<event_handler>
        self.evt.onPwm1(self)
        
    def onPwm2(self, event):  # wxGlade: p015.<event_handler>
        self.evt.onPwm2(self)

    def onPwm3(self, event):  # wxGlade: p015.<event_handler>
        self.evt.onPwm3(self)

    def onPwm4(self, event):  # wxGlade: p015.<event_handler>
        self.evt.onPwm4(self)

"""
