#!/usr/bin/env python
# P011

from sv3common import sv3, util_error, util_information
from time import sleep

# Global
EEsettings = [] # saves changes for writing to ee[rpm

ACK = '\006'
NACK = '\025'

# ------------------------------------------------------------------------------
# start up and close down
# Load frame with default values, from EEPROM
# ------------------------------------------------------------------------------
def onActivate(wnd):
    global EEsettings
    global i2cadr
    global bus
    i2cadr = int(wnd.adr)
    bus = wnd.sp
    # update status bar
    wnd.lblDevice.SetLabel(wnd.barStatus.GetStatusText(0))
    # remove com and baud
    wnd.barStatus.SetStatusText("",1)
    wnd.barStatus.SetStatusText("",2)
    # device ID
    dev = bus.i2c(i2cadr,[0xa1],2)
    v = dev[0]*256
    v += dev[1]
    wnd.barStatus.SetStatusText(str(v),3)
    wnd.barStatus.SetStatusText(str(i2cadr),3) # address
    
    # load eeprom values to 16
    EEsettings = []
    for j in range(16,32):
        v = bus.i2c(i2cadr,[0x90,j],1)
        EEsettings.append(str(v[0]))
    
    # 16,17,18 default back light values
    wnd.sldBL1.SetValue(int(EEsettings[0]))
    wnd.sldBL2.SetValue(int(EEsettings[1]))
    wnd.sldBL3.SetValue(int(EEsettings[2]))

# Close is important as this is a modal frame
def onClose(wnd,event):
    wnd.MakeModal(False)
    event.Skip()
    
# ******************************************************************************
# ******************************************************************************
# EVENTS

# ------------------------------------------------------------------------------
def onSoftreset(wnd):
    bus.i2c(i2cadr,[0x95],0)
    wnd.barStatus.SetStatusText("Reset complete")

# ------------------------------------------------------------------------------
def onHardreset(wnd):
    wnd.sp.SetDTR(True)
    sleep(0.1)
    wnd.sp.SetDTR(False)
    wnd.barStatus.SetStatusText("Reset complete")

# ------------------------------------------------------------------------------
# sends commaand 1 to clear screen 
def onCls(wnd):
    bus.i2c(i2cadr,[2,1],0)    
    #sv3(wnd.sp,wnd.adr+"c1\r") 
    
# ------------------------------------------------------------------------------
# helper for writing out a line
# command, string in list form, 13
def strList(s):
    l = [3] # command
    # add string
    for n in s:
        l.append(ord(n))
    l.append(13) # terminator
    return l    

# ------------------------------------------------------------------------------
# send button sends what ever is on lines to display. This works by sending
# a command and value first to get the cursor to the start of the line
# the values work for most displays being:
# Line1 0x80
# Line2 0xco
# Line3 0x94
# Line4 0xd4
def onSend(wnd):
    # Line 1    
    l = wnd.txtLine1.GetLineText(0)
    if len(l) !=0:
        bus.i2c(i2cadr,[2,128],0) # start of line
        bus.i2c(i2cadr,strList(l),0)
        #sv3(wnd.sp,wnd.adr+"c128\r") # start of line 1
        #sv3(wnd.sp,wnd.adr+"d"+l+"\r") # contents of line
    # Line 2    
    l = wnd.txtLine2.GetLineText(0)
    if len(l) !=0:
        bus.i2c(i2cadr,[2,192],0) # start of line
        bus.i2c(i2cadr,strList(l),0)
    # Line 3
    l = wnd.txtLine3.GetLineText(0)
    if len(l) !=0:
        bus.i2c(i2cadr,[2,148],0) # start of line
        bus.i2c(i2cadr,strList(l),0)
    # Line 4    
    l = wnd.txtLine4.GetLineText(0)
    if len(l) !=0:
        bus.i2c(i2cadr,[2,212],0) # start of line
        bus.i2c(i2cadr,strList(l),0)

# ------------------------------------------------------------------------------
# sliders alter the value for the rgb PWM signals, monochrome backlights 
# just use BL1 (red) 
# The default values are in EEsettings 
def onBL1(wnd):    
    EEsettings[0] = str(wnd.sldBL1.GetValue())
    bus.i2c(i2cadr,[1,EEsettings[0],EEsettings[1],EEsettings[2]],0)
#    sv3(wnd.sp,wnd.adr+"b"+EEsettings[0]+","+EEsettings[1]+","+EEsettings[2]+"\r")

def onBL2(wnd):    
    EEsettings[1] = str(wnd.sldBL2.GetValue())
    bus.i2c(i2cadr,[1,EEsettings[0],EEsettings[1],EEsettings[2]],0)

def onBL3(wnd):    
    EEsettings[2] = str(wnd.sldBL3.GetValue())
    bus.i2c(i2cadr,[1,EEsettings[0],EEsettings[1],EEsettings[2]],0)

# ------------------------------------------------------------------------------
# Alters eeprom contents to reflect text lines and also BL settings. To send
# a command preceed it by a 1 and finidh off with 0
def onSignon(wnd):
    ee = 32
    # Line 1    
    l = wnd.txtLine1.GetLineText(0)
    if len(l) !=0:
        bus.i2c(i2cadr,[0x91,ee,1],0) # command follows
        #sv3(wnd.sp,wnd.adr+"W"+str(ee)+",1\r") # command follows
        ee += 1
        sleep(0.1)
        bus.i2c(i2cadr,[0x91,ee,128],0)
        #sv3(wnd.sp,wnd.adr+"W"+str(ee)+",128\r") # start of line 1
        for n in l:
            ee += 1
            sleep(0.1)
            bus.i2c(i2cadr,[0x91,ee,ord(n)],0)
            #sv3(wnd.sp,wnd.adr+"W"+str(ee)+","+str(ord(n))+"\r") # contents of line
    
    # Line 2    
    l = wnd.txtLine2.GetLineText(0)
    if len(l) !=0:
        ee += 1
        bus.i2c(i2cadr,[0x91,ee,1],0) # command follows
        ee += 1
        sleep(0.1)
        bus.i2c(i2cadr,[0x91,ee,192],0)
        for n in l:
            ee += 1
            sleep(0.1)
            bus.i2c(i2cadr,[0x91,ee,ord(n)],0)

    # Line 3    
    l = wnd.txtLine3.GetLineText(0)
    if len(l) !=0:
        ee += 1
        bus.i2c(i2cadr,[0x91,ee,1],0) # command follows
        ee += 1
        sleep(0.1)
        bus.i2c(i2cadr,[0x91,ee,148],0)
        for n in l:
            ee += 1
            sleep(0.1)
            bus.i2c(i2cadr,[0x91,ee,ord(n)],0)

    # Line 4    
    l = wnd.txtLine4.GetLineText(0)
    if len(l) !=0:
        ee += 1
        bus.i2c(i2cadr,[0x91,ee,1],0) # command follows
        ee += 1
        sleep(0.1)
        bus.i2c(i2cadr,[0x91,ee,212],0)
        for n in l:
            ee += 1
            sleep(0.1)
            bus.i2c(i2cadr,[0x91,ee,ord(n)],0)

    ee += 1
    bus.i2c(i2cadr,[0x91,ee,0],0) # finish
    #sv3(wnd.sp,wnd.adr+"W"+str(ee)+",0\r") # finish
    
    # now Back light settings
    bus.i2c(i2cadr,[0x91,16,EEsettings[0]],0)
    bus.i2c(i2cadr,[0x91,17,EEsettings[1]],0)
    bus.i2c(i2cadr,[0x91,18,EEsettings[2]],0)
#    sv3(wnd.sp,wnd.adr+"W16,"+EEsettings[0]+"\r")
#    sv3(wnd.sp,wnd.adr+"W17,"+EEsettings[1]+"\r")
#    sv3(wnd.sp,wnd.adr+"W18,"+EEsettings[2]+"\r")

    wnd.barStatus.SetStatusText("EEPROM Updated",0)
    
# ******************************************************************************
# 
# ******************************************************************************

"""
class p017(wx.Frame):
    def __init__(self, type, wnd, com, *args, **kwds):
        # begin wxGlade: pi11.__init__
        kwds["style"] = wx.DEFAULT_FRAME_STYLE
        wx.Frame.__init__(self, *args, **kwds)

        if type == 'I':
            import p017_Ievents as events
        if type == 'S':
            import po17_events as events
        self.evt = events
        self.sp = com
        self.wnd = wnd
        self.adr = wnd.barStatus.GetStatusText(5)

        # Menu Bar

##binds
        self.Bind(wx.EVT_ACTIVATE, self.evt.onActivate(self))
        self.Bind(wx.EVT_CLOSE, self.onClose)

## events
    def onClose(self, event):  # wxGlade: bv4603.<event_handler>
        self.evt.onClose(self,event)

    def onSoftreset(self, event):  # wxGlade: bv4603.<event_handler>
        self.evt.onSoftreset(self)

    def onHardreset(self, event):  # wxGlade: bv4603.<event_handler>
        self.evt.onHardreset(self)

    def onSend(self, event):  # wxGlade: p017.<event_handler>
        self.evt.onSend(self)

    def onCls(self, event):  # wxGlade: p017.<event_handler>
        self.evt.onCls(self)

    def onBL1(self, event):  # wxGlade: p017.<event_handler>
        self.evt.onBL1(self)

    def onBL2(self, event):  # wxGlade: p017.<event_handler>
        self.evt.onBL2(self)

    def onBL3(self, event):  # wxGlade: p017.<event_handler>
        self.evt.onBL3(self)

    def onSignon(self,event):  # wxGlade: p017.<event_handler>
        self.evt.onSignon(self)

"""
