#!/usr/bin/env python
#

import wx
import os
from time import sleep
import sys
sys.path.append("device")
from sv3common import sv3, util_error, util_information, Connect, writeIni
# Baud rates on sv3 devices are coded inro single digits
BAUDRATES = {2400:'1',4800:'2',9600:'3',14400:'4',19200:'5',38400:'6',57600:'7',115200:'8'}

ACK = '\006'
NACK = '\025'
IACK = '\076' # inverted ACK

# bitmaps
bitmaps = {'Logo': "images/xLogo.ico", 'Refresh':"images/yRefesh.bmp", \
        'Connect':"images/Connected.ico", 'Disconnect':"images/Disconnected.ico" }

# contolling globals
sp = None   # serial port
activated = 0
address = "-1" # address of current device
EEdata = [] # keeps a list of changed addresses for EEPROM

# devices
from bv4601 import bv4601
from bv4603 import bv4603
from bv4614 import bv4614
from p011 import p011 # ADC
from p015 import p015 # GPIO
from p017 import p017 # LCD
Devices = {'406':bv4603,'4601':bv4601,'4603':bv4603,'4604':bv4603,'4614':bv4614,'11':p011, \
           '17':p017,'15':p015}
Descriptions = {'406':'This device is now the BV4604','4601':'Twin relay with ADC', \
                '4603':'Small Microstep motor driver','4604':'Larger Microstep motor driver', \
                '4614':'7 Segment LED Display','11':'ADC Chip','17':'LCD Display Chip', \
                '15':'GPIO with PWM'}

try:
     from serial.tools.list_ports import comports
except ImportError:
    comports = None

# ==============================================================================
# Part 2
# event handlers
# ==============================================================================

# ------------------------------------------------------------------------------
# on start up check to see if device and com port are already in place
# sv3.ini will exists if this has been run before
# ------------------------------------------------------------------------------
def onActivate(wnd):
    global activated
#    global currentStatus
    global address
    global sp
    activated = 1
    connected = False
    onRefreshport(wnd)  # populate com ports
    if os.path.isfile('sv3.ini'):
        # file exists get previous status
        try:
            inf = open('sv3.ini','r')
            s = inf.read()
        except:
            util_error("Problem with sws.ini file, - delete it and start again")
        else:
            if len(s) > 5:
                ss = s.rsplit(',')
                # update status bar
                wnd.barStatus.SetStatusText(ss[0],1) # com port
                wnd.barStatus.SetStatusText(ss[1],2) # Baud rate
                wnd.barStatus.SetStatusText(ss[2],5) # address
                address = ss[2]
                # see if com port is still there, will also set combo box
                if wnd.cboPort.SetStringSelection(ss[0]):
                    # try to connect to it
                    wnd.cboBaud.SetStringSelection(ss[1])
                    sp = Connect(sp,ss[0],ss[1])
                    if sp:
                        # com is still connected, see if device is
                        connected = True
                        wnd.cboPort.Enable(False)
                        wnd.cboBaud.Enable(False)
                        wnd.btnConnect.SetBitmapLabel(wx.Bitmap(bitmaps['Connect']))
                        onGetdevices(wnd)
                        if sv3(sp,address+"H\r") == ACK:
                            wnd.barStatus.SetStatusText('Connected ')
                            wnd.barStatus.SetStatusText(address,5)
                            wnd.btnDevice.Enable(True)
                            device = sv3(sp,address+"D\r")
                            version = sv3(sp,address+"V\r")
                            wnd.barStatus.SetStatusText(device,3)
                            wnd.barStatus.SetStatusText(version,4)
                        else:
                            wnd.barStatus.SetStatusText('Select a device from list ')

    if connected != True:
        wnd.barStatus.SetStatusText('Connect Com Port')

def onConnect(wnd):
    global sp
    global address
    port = wnd.cboPort.GetValue()
    baud = wnd.cboBaud.GetValue()
    wnd.barStatus.SetStatusText(port,1)
    wnd.barStatus.SetStatusText(baud,2)
    if activated:
        if sp: # disconnect  toggle
            sp = None
            wnd.btnConnect.SetBitmapLabel(wx.Bitmap(bitmaps['Disconnect']))
            wnd.cboPort.Enable(True)
            wnd.cboBaud.Enable(True)
            wnd.lstDevices.Clear()
        else: # connect toggle
            sp = Connect(sp,port,baud)
            if sp:
                connected = 1
                wnd.btnConnect.SetBitmapLabel(wx.Bitmap(bitmaps['Connect']))
                wnd.barStatus.SetStatusText('Conneced')
                address = '-1'
                wnd.cboPort.Enable(False)
                wnd.cboBaud.Enable(False)
                writeIni(sp,address)
            else:
                wnd.btnConnect.SetBitmapLabel(wx.Bitmap(bitmaps['Disconnect']))
                util_error("Cant open Port "+port+"Serial Port Error")
                wnd.cboPort.Enable(True)
                wnd.cboBaud.Enable(True)

def onRefreshport(wnd):
    wnd.cboPort.Clear()
    if comports:
        for port, desc, hwid in sorted(comports()):
                wnd.cboPort.Append(port)
        if wnd.cboPort.GetValue() == 'Select Port': # i.e. empty
            util_information("Port list returns blank, inserting ttyAMA0\rThis can be manually changed if required")
            wnd.cboPort.Append("/dev/ttyAMA0")
            wnd.cboPort.SetSelection(0)
    else:
        util_information("Unable to populate COM port list\rEnter the port name manualy\rPRi is probably/dev/ttyAMA0")

def onGetdevices(wnd):
    if sp: # only if connected
        wnd.lstDevices.Clear()
        wnd.barStatus.SetStatusText("Wait..")
        for j in range(97,123):
            #s = sv3(sp,chr(j)+"H\r")
            sp.flushInput()
            sp.write(chr(j)+"H\r")
            s = sp.read(1)
            # check for inverted device
            if s == IACK:
                util_information("Inverted output device "+chr(j)+" detected \nso setting it(them) for the correct polatity")
                sp.write(chr(j)+"I\r")
                s = ACK
            if s == ACK:
                device = sv3(sp,chr(j)+"D\r")
                version = sv3(sp,chr(j)+"V\r")
                wnd.lstDevices.Append("Addr:"+chr(j)+", ID:"+device+", Version:"+version)
        wnd.barStatus.SetStatusText("Finished searching for devices")

    else:
        util_error("Connect to COM port first")

# ------------------------------------------------------------------------------
# Double click list to select a particular device
# ------------------------------------------------------------------------------
def onList(wnd):
    global address
    n = wnd.lstDevices.GetSelection()
    s = wnd.lstDevices.GetString(n)
    ss = s.rsplit(',') # splits into catagories
    sss = ss[0].rsplit(':') # get actual address
    address = sss[1]
    wnd.barStatus.SetStatusText("Device address ("+address+") selected")
    ids = ss[1].rsplit(':') # id
    id = ids[1]
    wnd.barStatus.SetStatusText(id,3)
    sss = ss[2].rsplit(':') # version
    wnd.barStatus.SetStatusText(sss[1],4)
    wnd.barStatus.SetStatusText(address,5)
    wnd.btnDevice.Enable(True)
    writeIni(sp,address)
    wnd.lblSelected.SetLabel(Descriptions[id])

# ******************************************************************************
# CHANGE ADDRESS
def onNewaddress(wnd):
    if address != "-1":
        wnd.barStatus.SetStatusText("Wait..")
        newAddress = wnd.txtNewaddress.GetLineText(0)
        # location 1
        s = sv3(sp,address+"W1,"+str(ord(newAddress))+"\r")
        if s != ACK:
            util_error("Error writing to EEPROM address 1")
            return

        # location 14
        sleep(0.5)
        s = sv3(sp,address+"W14,"+str(ord(newAddress))+"\r")
        if s != ACK:
            util_error("Error writing to EEPROM address 14")
            return

        # location 250
        sleep(0.5)
        s = sv3(sp,address+"W250,"+str(ord(newAddress))+"\r")
        if s != ACK:
            util_error("Error writing to EEPROM address 14")
            return

        # restart device
        s = sv3(sp,address+"C\r")
        wnd.barStatus.SetStatusText("Address changed")
        util_information("Address changed, now use Get Attached Devices button again\r to see changes")

    else:
        util_error("A device needs selecting first ")

# ------------------------------------------------------------------------------
# Change Baud rate
# ------------------------------------------------------------------------------
def onBaud(wnd):
    try:
        br = BAUDRATES[int(wnd.txtBaud.GetLineText(0))]
    except:
        util_error("Baud rate "+wnd.txtBaud.GetLineText(0)+" is not available")
    else:
        wnd.barStatus.SetStatusText("Wait..")
        if sv3(sp,address+"W4,"+br+"\r") != ACK:
            util_error("Device reports error?\nChech that there is a device selected")
            wnd.barStatus.SetStatusText("Baud rate change error")
        else:
            sv3(sp,address+"C\r") # restart device
            wnd.barStatus.SetStatusText("Baud rate changed. Refresh list")
            wnd.lstDevices.Clear()

# ******************************************************************************
# EEPROM

# ------------------------------------------------------------------------------
# this is a tab change event. Idealy the tabshould be disabled until the
# user selects a device
# ------------------------------------------------------------------------------
def onEeprom(wnd):
    if address != "-1":
        print "okay"
    else:
        util_error("A device needs selecting first ")

# ------------------------------------------------------------------------------
# EEPROM refresh button
# Reads the whole of the EEPROM and loads it into the grid
# ------------------------------------------------------------------------------
def onEERefresh(wnd):
    wnd.barStatus.SetStatusText("Wait..")
    eestr = sv3(sp,address+"R0,256\r")
    elist = eestr.rsplit(",")
    row=0
    col=0
    #for cell in elist:
    for j in range(0, len(elist)-1):
        try:
            hexval = format(int(elist[j]),'02x')
        except:
            hexval = '??'
        else:        
            wnd.grdEeprom.SetCellValue(row,col, hexval )
            wnd.grdEeprom.SetCellTextColour(row, col, wx.BLACK)
            #print row, col, j, elist[j]
            col = col + 1
            if col > 7:
                col = 0
                row = row + 1
    wnd.barStatus.SetStatusText("")

# ------------------------------------------------------------------------------
# When the user changesthe contents of the EEPROM grid, that locationis kept
# in EEdata so that the change can be written to later. This method is much
# quicker and less error prone than writing the whole EEPROM data each time.
# ------------------------------------------------------------------------------
def onCellchange(wnd,event):
    row = event.GetRow()
    col = event.GetCol()
    wnd.grdEeprom.SetCellTextColour(row, col, wx.RED)
    EEdata.append((row*8)+col)

# ------------------------------------------------------------------------------
# EEPROM write button
# takes changed data as given in the list EEdata and writes the contents
# of the cell to EEPROM
# ------------------------------------------------------------------------------
def onEEWrite(wnd):
    wnd.barStatus.SetStatusText("Wait...")
    for j in EEdata:
        row = j / 8
        col = j % 8
        cell = wnd.grdEeprom.GetCellValue(row,col)
        #print j,row,col,cell
        wnd.grdEeprom.SetCellTextColour(row, col, wx.GREEN)
        # do actual write, device expects a decimal value so convert
        # the hex sring into a decimal string
        vs = str(int(cell,16))
        s = sv3(sp,address+"W"+str(j)+","+vs+"\r")
        if s != ACK:
            util_error("Error writing to EEPROM address "+cell)
            return
    wnd.grdEeprom.Refresh()
    wnd.barStatus.SetStatusText("Done...")

# ------------------------------------------------------------------------------
# Activates device window
# Calls window dependig on value in status bar, adding windows needs import
# and device dictionary updating
# ------------------------------------------------------------------------------
def onDevice(wnd):
    device =  wnd.barStatus.GetStatusText(3)
#    try:
    int(device) # in case its not a number
    frame_2 = Devices[device]('S',wnd, sp, None, wx.ID_ANY)
    frame_2.Show()
    frame_2.MakeModal(True)
#    except:
#        util_error("Device ID is not valid")

# ------------------------------------------------------------------------------
# have not got this working yet
# ------------------------------------------------------------------------------
def onClose(wnd):
    if activated:
        #control.Destroy()
        if sp:
            sp.close()

#code to put in frame class
"""
Replace these: (function __init__)
        self.btnConnect = wx.BitmapButton(self, wx.ID_ANY, wx.Bitmap(bitmaps['Disconnect'], wx.BITMAP_TYPE_ANY))
        self.btnRefresh = wx.BitmapButton(self, wx.ID_ANY, wx.Bitmap(bitmaps['Refresh'], wx.BITMAP_TYPE_ANY))
(function properties)
        _icon.CopyFromBitmap(wx.Bitmap(bitmaps['Logo'], wx.BITMAP_TYPE_ANY))
        
        
        self.grdEeprom.SetRowLabelValue(0, _("0x00"))
        self.grdEeprom.SetRowLabelValue(1, _("0x08"))
        self.grdEeprom.SetRowLabelValue(2, _("0x10"))
        self.grdEeprom.SetRowLabelValue(3, _("0x18"))
        self.grdEeprom.SetRowLabelValue(4, _("0x20"))
        self.grdEeprom.SetRowLabelValue(5, _("0x28"))
        self.grdEeprom.SetRowLabelValue(6, _("0x30"))
        self.grdEeprom.SetRowLabelValue(7, _("0x38"))
        self.grdEeprom.SetRowLabelValue(8, _("0x40"))
        self.grdEeprom.SetRowLabelValue(9, _("0x48"))
        self.grdEeprom.SetRowLabelValue(10, _("0x50"))
        self.grdEeprom.SetRowLabelValue(11, _("0x58"))
        self.grdEeprom.SetRowLabelValue(12, _("0x60"))
        self.grdEeprom.SetRowLabelValue(13, _("0x68"))
        self.grdEeprom.SetRowLabelValue(14, _("0x70"))
        self.grdEeprom.SetRowLabelValue(15, _("0x78"))
        self.grdEeprom.SetRowLabelValue(16, _("0x80"))
        self.grdEeprom.SetRowLabelValue(17, _("0x88"))
        self.grdEeprom.SetRowLabelValue(18, _("0x90"))
        self.grdEeprom.SetRowLabelValue(19, _("0x98"))
        self.grdEeprom.SetRowLabelValue(20, _("0xA0"))
        self.grdEeprom.SetRowLabelValue(21, _("0xA8"))
        self.grdEeprom.SetRowLabelValue(22, _("0xB0"))
        self.grdEeprom.SetRowLabelValue(23, _("0xB8"))
        self.grdEeprom.SetRowLabelValue(24, _("0xC0"))
        self.grdEeprom.SetRowLabelValue(25, _("0xC8"))
        self.grdEeprom.SetRowLabelValue(26, _("0xD0"))
        self.grdEeprom.SetRowLabelValue(27, _("0xD8"))
        self.grdEeprom.SetRowLabelValue(28, _("0xE0"))
        self.grdEeprom.SetRowLabelValue(29, _("0xE8"))
        self.grdEeprom.SetRowLabelValue(30, _("0xF0"))
        self.grdEeprom.SetRowLabelValue(31, _("0xF8"))

Add these
        self.Bind(wx.EVT_ACTIVATE, sv3evt.onActivate(self))
#        self.Bind(wx.EVT_CLOSE, sv3utils.onClose(self)

    def onConnect(self, event):  # wxGlade: MyFrame.<event_handler>
        sv3evt.onConnect(self)

    def onRefreshport(self, event):  # wxGlade: MyFrame.<event_handler>
        sv3evt.onRefreshport(self)

    def onGetdevices(self, event):  # wxGlade: MyFrame.<event_handler>
        sv3evt.onGetdevices(self)

    def onDevice(self, event):  # wxGlade: MyFrame.<event_handler>
        sv3evt.onDevice(self) # pass the list

    def onList(self, event):  # wxGlade: MyFrame.<event_handler>
        sv3evt.onList(self)

    def onNewaddress(self, event):  # wxGlade: MyFrame.<event_handler>
        sv3evt.onNewaddress(self)

    def onEERefresh(self, event):  # wxGlade: MyFrame.<event_handler>
        sv3evt.onEERefresh(self)

    def onEEWrite(self, event):  # wxGlade: MyFrame.<event_handler>
        sv3evt.onEEWrite(self)

    def onEeprom(self, event):  # wxGlade: MyFrame.<event_handler>
        sv3evt.onEeprom(self)

    def onCellchange(self, event):  # wxGlade: MyFrame.<event_handler>
        sv3evt.onCellchange(self, event)

    def onBaud(self, event):  # wxGlade: MyFrame.<event_handler>
        sv3evt.onBaud(self)
"""